if {[regexp {[]\(\)\"\{\}\[]} $HOME]} {alertnote "Alpha may not work properly if it's pathname includes parenthesis, square brackets, double quotes, or curly braces."}

# Get name Alpha's current name.
foreach p [processes] {
	if {[lindex $p 1] == "ALFA"} {
		set ALPHA [lindex $p 0]
		break
	}
}

set optionIsMeta 	1
set emacsBindings	1

# Icon menu names. The number refers to the resource id of the #ics family.
#set fsetMenuName 	"SetFiles"
set fsetMenuName 	"129"
set helpMenu		"130"
set winMenu 		"263"

set tclMenu			"269"
set thinkMenu 		"300"
set cwarrierMenu	"268"
set cwdebugMenu		"274"
set latexMenu		"270"
set thinkRefMenu	"265"
set perlMenu 		"132"
set terrMenu		"271"
set toolserverMenu	"133"

# A few definitions I'll need later.
proc quoteExpr str {
	regsub -all {\\} $str {\\\\} str
	regsub -all {\|} $str {\|} str
	regsub -all {\*} $str {\\*} str
	regsub -all {\+} $str {\\+} str
	regsub -all {\(} $str {\\(} str
	regsub -all {\)} $str {\\)} str
	regsub -all {\[} $str {\\[} str
	regsub -all {\]} $str {\\]} str
	return $str
}

proc quoteExpr2 str {
	regsub -all {\\} $str {\\\\} str
	regsub -all {\|} $str {\|} str
	regsub -all {\*} $str {\\*} str
	regsub -all {\+} $str {\\+} str
	regsub -all {\(} $str {\\(} str
	regsub -all {\)} $str {\\)} str
	regsub -all {\{} $str "\\\{" str
	regsub -all {\}} $str "\\\}" str
	regsub -all {\[} $str {\\[} str
	regsub -all {\]} $str {\\]} str
	return $str
}

source ":Tcl:SystemCode:menus.tcl"

if {$emacsBindings} {
	source ":Tcl:SystemCode:emacs.tcl"
} else {
	source ":Tcl:SystemCode:vi.tcl"
}


bind Left  backwardChar
bind Left <c> beginningOfLine
bind Left <s> backwardCharSelect
bind Left <sc> beginningLineSelect
bind Left <z> {scrollLeftCol 15}
bind Left <o> backwardWord
bind Left <os> backwardWordSelect

bind Right  forwardChar
bind Right <c> endOfLine
bind Right <s> forwardCharSelect
bind Right <sc> endLineSelect
bind Right <z> {scrollRightCol 15}
bind Right <o> forwardWord
bind Right <os> forwardWordSelect

bind Up  previousLine
bind Up <s> prevLineSelect
bind Up <c> beginningOfBuffer
bind Up <sc> beginningBufferSelect
bind Up <z> scrollUpLine
bind Up <o> scrollUpLine

bind Down  nextLine
bind Down <c> endOfBuffer
bind Down <s> nextLineSelect
bind Down <sc> endBufferSelect
bind Down <z> scrollDownLine
bind Down <o> scrollDownLine

bind ',' <z> pushMark
bind '.' <z> popMark

# Keypad definitions
bind KPad4  backwardWord 				
bind KPad4   <c> backwardDeleteWord 
bind KPad6   forwardWord 				
bind KPad6   <c> deleteWord 
bind Clear   insertToTop 				
# Never bind Keypad /
# Never bind Keypad *
bind KPad0   pageBack 					
bind Enter   pageForward
;bind KPad3   repeatSearchForward 		
;bind KPad1   repeatSearchBackward 	
bind Kpad1 prevFunc
bind Kpad3 nextFunc
bind KPad.   endOfBuffer 				
bind KPad5   exchangePointAndMark 	
bind KPad7   backwardDeleteWord 		
bind KPad9   deleteWord 				
bind 's' <z> 	isearch			
bind 'r' <z> 	rsearch			
bind 'a' <z> 	beginningOfLine	
bind 'b' <z> 	backwardChar		
bind 'd' <z> 	deleteChar		
bind 'f' <z> 	forwardChar		
bind 'e' <z> 	endOfLine		
bind 'g' <z> 	abortEm			
bind 'k' <z> 	killLine			
bind 'l' <z>	centerRedraw
bind 't' <z> 	insertToTop		
bind 'p' <z> 	previousLine		
bind 'n' <z> 	nextLine			
bind 'o' <z> 	openLine			
bind 'u' <z> 	iterationCount	
bind 'v' <z> 	pageForward		
bind 'v' <e> 	pageBack
bind 'z' <z> 	pageBack
bind 'w' <z> 	cut			
bind 'w' <e> 	copy
bind "' '" <z> 	setMark
bind 'y' <z> 	yank				
bind '1' <z>	execAbbrev

# Alpha variable and flag definitions.
set temp [getMainDevice]
set tileWidth [expr {[lindex $temp 2] - [lindex $temp 0] - 10}]
if {$tileWidth > 510} {set defWidth 510} else {set defWidth $tileWidth }
set tileHeight [expr [lindex $temp 3] - [lindex $temp 1] - 62]
set defHeight $tileHeight
unset temp

source ":Tcl:SystemCode:definitions.tcl"


bind F1     	iconify 					
#bind  <c> F1 	wordCompletion 
bind F2     	cut 						
bind F3     	copy 						
bind F4     	yank 						
bind F5     	prevWindow
bind F6			nextWindow
bind F7			tiled
bind F8			swapWithNext
bind F9     	pushMark
bind F10    	popMark
bind F11    	shrinkHigh
bind F12    	shrinkLow
bind F13    	startKeyboardMacro 		
bind F14    	endKeyboardMacro 			
bind F15    	executeKeyboardMacro 		

bind Del    	deleteChar 				
bind Help   	alphaHelp 					
bind Home   	beginningOfBuffer 			
bind End    	endOfBuffer 				
bind Pgup   	pageBack 					
bind Pgdn   	pageForward  				

proc debug {} {
	uplevel #0 {
		set debugging 1
	}
}

# Load electric alias, rebind tcl file completion for precedence.
proc loadElectricAlias {} {
	global HOME
	uplevel #0 {
		source "$HOME:Tcl:ElectricAlias:electricAlias.tcl"
	}
	message "ElectricAlias loaded."
	bind '\t' tclFileCompletion "Csh"
	enableMenuItem -m install "Electric Alias" off
}


source "$HOME:Tcl:SystemCode:procs.tcl"
source "$HOME:Tcl:SystemCode:flags.tcl"
source "$HOME:Tcl:SystemCode:modes.tcl"
source "$HOME:Tcl:SystemCode:filesets.tcl"
source "$HOME:Tcl:SystemCode:copyRing.tcl"
source "$HOME:Tcl:SystemCode:latexMode.tcl"
source "$HOME:Tcl:SystemCode:bibtexMode.tcl"
set auto_path [list $HOME:Tcl:SystemCode $HOME:Tcl:UserCode]
source ":Tcl:SystemCode:library.tcl"

# Use "userStartup.tcl" to define or change any tcl information. */
if {[catch {source ":userStartup.tcl"}]} {
	alertnote "An error occurred while loading UserStartup.tcl"
}

# source {PowerPete:Desktop Folder:down:phm:modeprocs.tcl}
# source {PowerPete:Desktop Folder:down:phm:modes.tcl}


# Should be last so all filesets make it in.
makeFilesetMenu

menu -n choose -m -p changeFileSet [lsort [array names fileSets]]
markMenuItem -m choose $currFileSet on

set modes [lsort $modes]
buildFlagsVars

if {![info exists initchoice]} {
	if {[askyesno "Do you use a U.S. keyboard?"] == "yes"} {
		set res 0
	} else {set res 1}

	addUserLine "\r\# The next two lines set up your choice for US vs international keyboard."
	addUserLine "\# If you want to change this choice, remove the lines, restart Alpha, and"
	addUserLine "\# register your new choice."
	addUserLine "set initchoice 1"
	addUserLine "init $res 2\r"
	init $res 2
}

message "Initialization Complete"
