#=============================================================================
#	Window handling routines. All procs are bound in AlphaBits.tcl.
#=============================================================================

proc shrinkHigh {} {
	global tileTop
	set text [getGeometry]
	set left [lindex $text 0]
	set top [lindex $text 1]
	set width [lindex $text 2]
	sizeWin $width 150
	moveWin $left $tileTop
}

proc shrinkLow {} {
	set text [getGeometry]
	set left [lindex $text 0]
	set top [lindex $text 1]
	set width [lindex $text 2]
	moveWin $left 315
	sizeWin $width 146
}

proc shrinkFull {} {
	global tileTop tileHeight
	moveWin 2 $tileTop
	sizeWin 510 $tileHeight
}

proc shrinkLeft {} {
	global tileWidth tileTop tileHeight
	
	set margin 4
	set width [expr ($tileWidth/2)-$margin]
	set text [getGeometry]
	set width [expr ($tileWidth/2)-$margin]
	set width [expr {$width + $margin / 2}]
	moveWin 2 $tileTop
	sizeWin $width $tileHeight
}

proc shrinkRight {} {
	global tileWidth tileTop tileHeight
	
	set margin 4
	set width [expr ($tileWidth/2)-$margin]
	set text [getGeometry]
	set width [expr ($tileWidth/2)-$margin]
	set width [expr {$width + $margin / 2}]
	moveWin [expr 2 + $width + $margin] $tileTop
	sizeWin $width $tileHeight
}

proc swapWithNext {} {
	set files [winNames -f]
	if {[llength $files] < 2} return
	bringToFront [lindex $files 1]
}
	


proc nextWindow {} {
	global winActive 
	set files [winNames -f]
	if {[llength $files] < 2} {return}
	set f [lindex $files 0]
	set aind [lsearch $winActive $f]
	if {$aind < 0} {error "No win '$f'"}
	set rng [lrange $winActive 0 [expr $aind-1]]
	set winActive [concat [lrange $winActive $aind end] $rng]
	set winActive [lrange $winActive 1 end]
	lappend winActive $f
	bringToFront [lindex $winActive 0]
}


proc prevWindow {} {
	global winActive 
	set files [winNames -f]
	if {[llength $files] < 2} {return}
	set f [lindex $files 0]
	set aind [lsearch $winActive $f]
	if {$aind < 0} {error "No win '$f'"}
	set rng [lrange $winActive 0 [expr $aind-1]]
	set winActive [concat [lrange $winActive $aind end] $rng]
	set f2 [lindex [lrange $winActive end end] 0]
	set winActive [lreplace $winActive end end]
	set winActive [linsert $winActive 0 $f2]
	bringToFront $f2
}

proc dispFullName {} {
	message [lindex [winNames -f] 0]
}

proc vertically {} {
	global tileHeight tileTop tileWidth
	global numWinsToTile
	set margin 22
	set names [winNames -f]
	set numWins [llength $names]
	if ($numWins<=1) return
	if ($numWins>$numWinsToTile) {set numWins $numWinsToTile}
	set height [expr ($tileHeight/$numWins)-$margin]
	set height [expr {$height + $margin / $numWins}]
	set width $tileWidth
	set ver $tileTop
	if {$numWins == 0} {return}

	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] 1000 0
		sizeWin [lindex $names $i] $width $height
	}

	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] 3 $ver
		set ver [expr $ver+$margin+$height]
	}
}

proc horizontally {} {
	global tileHeight tileWidth tileTop numWinsToTile

	set names [winNames -f]
	set numWins [llength $names]
	if ($numWins<=1) return
	if ($numWins>$numWinsToTile) {set numWins $numWinsToTile}
	set margin 4
	set width [expr ($tileWidth/$numWins)-$margin]
	set width [expr {$width + $margin / $numWins}]
	set height $tileHeight
	set hor 2
	if {$numWins == 0} {return}

	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] 1000 0
		sizeWin [lindex $names $i] $width $height
	}

	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] $hor $tileTop
		set hor [expr $hor+$width+$margin]
	}
}

proc tiled {} {
	global tileHeight tileWidth numWinsToTile tileTop
	set xPan 8
	set yPan 10
	set xMarg 3
	set yMarg $tileTop
	set yMax 50
	set names [winNames -f]
	set numWins [llength $names]
	if ($numWins<1) return
	set line 0	
	set height [expr $tileHeight-$yPan*($numWins-1)]
	set width [expr $tileWidth-$xPan*($numWins-1)]
	
	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] [expr $xMarg+$i*$xPan] [expr $yMarg+$line]
		set line [expr $line+$yPan]
		if ($line>$yMax) {set line 0}
		sizeWin [lindex $names $i] $width $height
	}
}


proc overlay {} {
	global defHeight defWidth numWinsToTile tileTop
	set names [winNames -f]
	set numWins [llength $names]
	if ($numWins<1) return
	for {set i 0} {$i < $numWins} {incr i} {
		moveWin [lindex $names $i] 3 $tileTop
		sizeWin [lindex $names $i] $defWidth $defHeight
	}
}


proc threeQuarters {} {
	global tileHeight tileWidth tileTop

	if {[llength [set nms [winNames -f]]] <= 2} return
	set one [lindex $nms 0]
	set two [lindex $nms 1]
	set margin 22
	set height [expr ($tileHeight - $margin) / 4]

	moveWin $one 1000 0
	sizeWin $one $tileWidth [expr 3 * $height]
	moveWin $two 1000 0
	sizeWin $two $tileWidth $height

	set ver $tileTop
	moveWin $one 3 $ver
	moveWin $two 3 [expr $ver + 3 * $height + $margin]
}
bind '3' <Q> threeQuarters


proc chooseAWindow {} {
    set name [listpick [lsort -ignore [winNames]]]
    if {[string length $name]} {
    	bringToFront $name
		if [icon -q] { icon -f $name -o }
   	}
}


proc winComp {curr c} {
	if {$c != "\t"} {return $c}
	
	set matches {}
	foreach w [winNames] {
		if {[string match "$curr*" $w]} {
			lappend matches $w
		}
	}
	if {![llength $matches]} {
		beep
	} elseif {[llength $matches] == 1} {
		return [string range [lindex $matches 0] [string length $curr] end]
	}
	return ""
}

proc nextWin {} {
	global winActive 
	set files [winNames -f]
	if {[llength $files] < 2} {return ""}
	set f [lindex $files 0]
	set aind [lsearch $winActive $f]
	if {$aind < 0} {error "No win '$f'"}
	if {[incr aind] < [llength $winActive]} {
		return [file tail [lindex $winActive $aind]]
	} else {
		return [file tail [lindex $winActive 0]]
	}
}

proc chooseWindowStatus {} {
	if {[llength [winNames]] < 2} {message "no other window."; return}
	set next [nextWin]
	set res [statusPrompt "Goto window: ($next):" winComp]
	if {[string length $res]} {
		catch {bringToFront $res}
	} else {
		catch {bringToFront $next}
	}
}
bind f9 chooseWindowStatus

proc iconify {} { 
	icon -t 
}



proc zoom {} {
	global nzmState tileHeight tileWidth zoomedGeo tileTop
	
	set win [lindex [winNames -f] 0]
	if {[info exists nzmState($win)]} {
		if {[getGeometry] == $zoomedGeo} {
			set state $nzmState($win)
			moveWin [lindex $state 0] [lindex $state 1]
			sizeWin [lindex $state 2] [lindex $state 3]
			unset nzmState($win)
			return
		}
	} 

	set nzmState($win) [getGeometry]
	moveWin 3 $tileTop
	sizeWin $tileWidth $tileHeight

	if {![info exists zoomedGeo]} {
		set zoomedGeo [getGeometry]
	}
}
