/***
 * URLHelperComponentPrivate.h
 *
 *  Header file for private URLHelper component stuff -- things that only debugging routines
 *  and the URLHelper component itself will need to include
 *
 ***/

#pragma once
#include "URLHelperComponent.h"
#include "MemoryBuffer.h"

/**
 ** The global storage for this component
 **/

typedef struct URLHMirror  URLHMirror, *URLHMirrorPtr, **URLHMirrorHandle;

struct URLHMirror{
	URLHMirrorPtr	next;					// Linked list!

	short			oName;					// Name of this mirror
	URLHMirrorFlags	mirrorFlags;			// Usage flags
	
};


typedef struct URLHMirrorList URLHMirrorList, 
		*URLHMirrorListPtr, **URLHMirrorListHandle;

struct URLHMirrorList {
	Handle				stringBuffer;		// strings for our use!
	URLHMirrorListPtr	next;				// Next dude in the list of mirror lists
	URLHMirrorPtr		firstMirror;		// First mirror in the list

	short		oName;						// Name of the mirror list

	long		listStatus;					// Flags for the list

};

typedef struct {
	ComponentInstance	self;
    URLHMirrorListPtr	mirrorList;
} URLGlobals, *URLGlobalsPtr, **URLGlobalsHandle;

typedef struct URLHParseState *URLHParseStatePtr, **URLHParseStateHandle;

/**
 ** Status flags
 **/

#define kMLModified		0x01

/**
 ** The revision level of this component
 **/

enum {
	URLHelperInterfaceRevision = 0x00010001
};

/**
 ** If we are running in debug mode, then we should define the dispatch routine.
 **/

#ifdef DEBUG_IT
	pascal ComponentResult URLHelperDispatcher (ComponentParameters	*params,
											Handle				storage);
#endif

/**
 ** Protos for the routines that do the real work
 **/

pascal	ComponentResult	_URLHelperOpen		(Handle					storage,
										 ComponentInstance		self);
enum {
	uupURLOpen = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(ComponentInstance)))
};

pascal	ComponentResult	_URLHelperClose		(Handle					storage,
										 ComponentInstance		self);
enum {
	uupURLClose = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(ComponentInstance)))
};

pascal	ComponentResult	_URLHelperCanDo		(short					selector);
enum {
	uupURLCanDo = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short)))
};

pascal	ComponentResult	_URLHelperVersion	(void);
enum {
	uupURLVersion = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
};

pascal	ComponentResult	_URLHelperTarget		(Handle					storage,
										 ComponentInstance		capturingComponent);
enum {
	uupURLTarget = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(ComponentInstance)))
};

/**
 ** Our own routines
 **/

pascal ComponentResult _URLHelperGetMirrorList (Handle storage,
										StringPtr			string,
										URLHMirrorListPtr	*ref);
enum {
	uupURLHelperGetMirrorList = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(StringPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(URLHMirrorListPtr)))
};

pascal ComponentResult _URLHelperNewMirrorList (Handle storage,
										StringPtr			string,
										URLHMirrorListPtr	*ref);
enum {
	uupURLHelperNewMirrorList = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(StringPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(URLHMirrorListPtr)))
};

pascal ComponentResult _URLHelperAddMirror (Handle storage,
										URLHMirrorListPtr	ref,
										StringPtr			mirrorName,
										URLHMirrorFlags		flags);
enum {
	uupURLHelperAddMirror = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(URLHMirrorListPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(StringPtr)))
		| STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(URLHMirrorFlags)))
};

pascal ComponentResult _URLHelperNumberOfMirrors (Handle storage,
										URLHMirrorListPtr	ref,
										short				*num);
enum {
	uupURLHelperNumberOfMirrors = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(URLHMirrorListPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(short *)))
};

pascal ComponentResult _URLHelperGetUsableMirror (Handle storage,
										URLHMirrorListPtr	mirrorListRef,
										URLHMirrorPtr		*mirrorRef,
										short				index);
enum {
	uupURLHelperGetUsableMirror = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(URLHMirrorListPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(URLHMirrorPtr *)))
		| STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(short)))
};

pascal ComponentResult _URLHelperNewParseState (Handle storage,
										StringPtr			aURL,
										URLHParseStatePtr	*pState);
enum {
	uupURLHelperNewParseState = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(StringPtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(URLHParseStatePtr *)))
};

pascal ComponentResult _URLHelperDisposeParseState (Handle storage,
										URLHParseStatePtr	pState);
enum {
	uupURLHelperDisposeParseState = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(URLHParseStatePtr)))
};

pascal ComponentResult _URLHelperGetMirrorRep (Handle storage,
										URLHParseStatePtr	pState,
										StringPtr			newURL,
										short				index);
enum {
	uupURLHelperGetMirrorRep = kPascalStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(ComponentResult)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(Handle)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(URLHParseStatePtr)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(StringPtr)))
		| STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(short)))
};

/**
 ** A little something to help out with our life
 **/
#define genericError	-1L

/**
 ** Utility routines that we use
 **/
