/********************************************
 **** Core Class Demo by Yves Schmid
 ****
 **** Tree.cp
 ****
 **** Author:		Yves Schmid  
 **** Created:      14 August 1994
 **** Modified:     14 August 1994
 **** Version:      1.0
 **** Compatible:   C++
 ****
 **** Description:  Tree structure demo
 ****
 *************************/


#include "CoreHead.h"

#include <stdio.h>
#include <string.h>

//*************************************


const long cmd_printmyself	= 1024;	// A command which asks the stringnode to print its string


//*************************************
// A simple CoreHead which stocks and prints a string

class stringnode : public CoreHead
{
	char	data[256];


	protected:


	// Override the "receivecmd" to handle the "cmd_printmyself" command:

	void receivecmd(long cmd, void *info);

	public:

	// Constructors

	stringnode(char *str, CoreHead *supervisor);	// Object with a supervisor
	stringnode(char *str);							// Unlinked object
	
	// Destructor
	
	~stringnode();
};

//..............................

stringnode::stringnode(char *str, CoreHead *supervisor) : CoreHead(supervisor,0,1)	
											// Initialize CoreHead with the supervisor,
											// destination entry 0 and 1 entry in the new
											// created object 
											 
{
	newlevel();			// A new heritage level!
	strcpy(data,str);	// Copy string
}

//..............................


stringnode::stringnode(char *str) : CoreHead(1)	// Unlinked object with one entry 
{
	newlevel();			// A new heritage level!
	strcpy(data,str);	// Copy string
}

//..............................

stringnode::~stringnode()
{
	printf("%s deleted\n", data);
}

//..............................


void stringnode::receivecmd(long cmd, void *info)
{
	switch(cmd)
	{
		case cmd_printmyself:	// Ok it is the right message, I can print the string.
		printf("%s\n",data);	
		break;
		
		default:				// Unknow message? Call the parent class
		CoreHead::receivecmd(cmd,info);
	}
}

//..............................


//*******************************************************************


main()
{
	// Creating a tree:
	//
	//		  A
	//		/   \
	//	   B1	 B2
	//	  / \	/ \
	//	 C1 C2 D1  D2

	printf("*** CCL Tree demo ***\n\n");
	printf("Creating a tree:\n\n");
	printf("         A			\n");
	printf("       /   \\		\n");
	printf("      B1    B2		\n");
	printf("     /\\     /\\	\n");
	printf("    C1 C2  D1 D2	\n");

	//*****************************

	stringnode *a,*b1,*b2,*c1,*c2,*d1,*d2;

	a = new stringnode("A");
	b1 = new stringnode("B1",a);
	b2 = new stringnode("B2",a);
	c1 = new stringnode("C1",b1);
	c2 = new stringnode("C2",b1);
	d1 = new stringnode("D1",b2);
	d2 = new stringnode("D2",b2);

	//*****************************


	printf("\n\n* Scan the tree using the \"A\" node:\n");
	a->docmd(cmd_printmyself,CCF_EVERYWHERE);

	printf("\n\n* Scan the tree using the \"D2\" node:\n");
	d2->docmd(cmd_printmyself,CCF_EVERYWHERE);


	//*****************************

	printf("\n\n* Print \"D1\" and its ascendant nodes\n");
	d1->docmd(cmd_printmyself,CCF_ASCENDANT);

	printf("\n\n* Print \"B2\" and its descendant nodes\n");
	b2->docmd(cmd_printmyself,CCF_DESCENDANT);

	//*****************************

	printf("\n\n* Delete the \"B1\" node:\n");
	delete b1;

	printf("\n\n* Delete the \"A\" node:\n");
	delete a;	
}


